use std::sync::Arc;

use serde::{Serialize, Deserialize};
use axum::{Extension, extract::Request, body::Body, middleware::Next, response::{IntoResponse, Response}, http::{header::AUTHORIZATION, StatusCode}, Json};
use jsonwebtoken::{decode, Header};
use tracing::{debug, warn, error};

use crate::{state::ApiState, api::{AppId, AppDescription}};

#[derive(Serialize, Deserialize, Debug)]
struct JWTClaims {
    sub: AppId,
}


pub(super) async fn auth(s: Extension<Arc<ApiState>>, mut request: Request<Body>, next: Next) -> Response {
    if let Some(auth_header) = request.headers().get(AUTHORIZATION) {
        if let Ok(header_string) = auth_header.to_str() {
            if header_string.starts_with("Bearer") {
                if let Ok(token) = decode::<JWTClaims>(
                    &header_string[7..],
                    s.jwt_decoding_key(),
                    s.jwt_validation(),
                ) {
                    if let Ok(true) = s.app_exists(&token.claims.sub) {
                        debug!("Authentication for {:?} succeeded.", &token.claims.sub);
                        request.extensions_mut().insert(token.claims.sub);
                        return next.run(request).await;
                    }
                }
            }
        }
    }
    debug!("Authentication failed for request '{:?}'.", &request);
    StatusCode::UNAUTHORIZED.into_response()
}



pub(super) async fn register(s: Extension<Arc<ApiState>>, Json(data): Json<AppDescription>) -> Response {
    // Maybe ask for consent by user

    // If user wants registration, proceed
    let result = s.add_app(&data);

    match result {
        Ok(id) => {
            // Build JWT, respond
            let jwt = jsonwebtoken::encode(
                &Header::default(),
                &JWTClaims {sub: id},
                &s.jwt_encoding_key(),
            );
            match jwt {
                Ok(token) => (StatusCode::OK, token).into_response(),
                Err(e) => {
                    warn!("Failed to encode token: {:?}", e);
                    StatusCode::INTERNAL_SERVER_ERROR.into_response()
                }
            }
        },
        Err(e) => {
            error!("Failed to register new application! {:?}", e);
            StatusCode::INTERNAL_SERVER_ERROR.into_response()
        },
    }
}